@@ -145,6 +145,25 @@ struct TinyAlgebra {
     return res;
   }
 
+  TINY_INLINE static MatrixX mult(const MatrixX &a,
+                                  const MatrixX &b) {
+    return a * b;
+  }
+
+  TINY_INLINE static VectorX mult(const MatrixX &a,
+                                  const VectorX &b) {
+    return a * b;
+  }
+  TINY_INLINE static Vector3 mult(const MatrixX &a,
+                                  const Vector3 &b) {
+    VectorX vx = VectorX(3);
+    vx[0] = b.getX();
+    vx[1] = b.getY();
+    vx[2] = b.getZ();
+    vx = mult(a, vx);
+    return Vector3(vx[0], vx[1], vx[2]);
+  }
+
   template <typename T1, typename T2>
   TINY_INLINE static auto dot(const T1 &vector_a, const T2 &vector_b) {
     return vector_a.dot(vector_b);
@@ -242,6 +261,16 @@ struct TinyAlgebra {
    * Returns a 3x3 identity matrix.
    */
   TINY_INLINE static Matrix3 eye3() { return diagonal3(TinyConstants::one()); }
+
+  TINY_INLINE static MatrixX eye(int n) {
+    MatrixX mat = create_matrix_x(n, n);
+    mat.set_zero();
+    for (int i = 0; i < n; i++) {
+      mat(i, i) = one();
+    }
+    return mat;
+  }
+
   TINY_INLINE static void set_identity(Quaternion &quat) {
     quat.set_identity();
   }
@@ -259,6 +288,8 @@ struct TinyAlgebra {
   }
   TINY_INLINE static Scalar pi() { return TinyConstants::pi(); }
 
+  TINY_INLINE static Scalar half_pi() { return TinyConstants::half_pi(); }
+
   TINY_INLINE static Scalar scalar_from_string(const std::string &s) {
     return TinyConstants::scalar_from_string(s);
   }
@@ -565,20 +596,33 @@ struct TinyAlgebra {
    * Computes the quaternion delta given current rotation q, angular velocity w,
    * time step dt.
    */
-  TINY_INLINE static Quaternion quat_velocity(const Quaternion &q,
+  
+	TINY_INLINE static Quaternion quat_velocity(const Quaternion &q,
                                               const Vector3 &w,
                                               const Scalar &dt) {
-    return w * q * (dt * half());
-    // Quaternion delta(q[3] * w[0] + q[1] * w[2] - q[2] * w[1],
-    //                  q[3] * w[1] + q[2] * w[0] - q[0] * w[2],
-    //                  q[3] * w[2] + q[0] * w[1] - q[1] * w[0],
-    //                  -q[0] * w[0] - q[1] * w[1] - q[2] * w[2]);
-    // delta *= half() * dt;
-    // // print("delta quat", delta);
-    // return delta;
-  }
+        auto ww = (-q.x()*w[0] - q.y()*w[1] - q.z()*w[2]  ) * (half() * dt);
+        auto xx = ( q.w()*w[0] + q.z()*w[1] - q.y()*w[2]) * (half() * dt);
+        auto yy = ( q.w()*w[1] + q.x()*w[2] - q.z()*w[0]) * (half() * dt);
+        auto zz = ( q.w()*w[2] + q.y()*w[0] - q.x()*w[1]) * (half() * dt);
 
+        Quaternion delta = quat_from_xyzw_unsafe(xx,yy,zz, ww);
+        return delta;
+    }
 
+    TINY_INLINE static Quaternion quat_velocity_spherical(const Quaternion &q,
+                                              const Vector3 &vel,
+                                              const Scalar &dt) {
+        //return w * q * (dt * half());
+        auto w = (-q.x() * vel[0] - q.y() * vel[1] - q.z() * vel[2]) * (half() * dt);
+        auto x = (q.w() * vel[0] + q.y() * vel[2] - q.z() * vel[1]) * (half() * dt);
+        auto y = (q.w() * vel[1] + q.z() * vel[0] - q.x() * vel[2]) * (half() * dt);
+        auto z = (q.w() * vel[2] + q.x() * vel[1] - q.y() * vel[0]) * (half() * dt);
+        //we may create a zero length quaternion, use the 'unsafe' version
+        Quaternion delta = quat_from_xyzw_unsafe(x,y,z,w);
+        return delta;
+    }
+
+ 
   TINY_INLINE static Quaternion quat_integrate(const Quaternion &q,
       const Vector3 &ang_vel,
       const Scalar &dt){
@@ -614,6 +658,18 @@ struct TinyAlgebra {
                                                      const Scalar &w) {
     return Quaternion(x, y, z, w);
   }
+
+  //unsafe version allows zero-length quaternion x=y=z=w=0
+  TINY_INLINE static const Quaternion quat_from_xyzw_unsafe(const Scalar &x,
+                                                     const Scalar &y,
+                                                     const Scalar &z,
+                                                     const Scalar &w) {
+    Quaternion q;
+    q.setValue(x,y,z,w);
+    return q;
+  }
+  
+
   
   /**@brief Set the quaternion using euler angles, compatible with PyBullet/ROS/Gazebo
    * @param yaw Angle around Z
@@ -724,10 +780,14 @@ struct TinyAlgebra {
     return TinyConstants::sin1(s);
   }
 
+  TINY_INLINE static Scalar asin(const Scalar& s) {
+    return TinyConstants::asin(s);
+  }
+
   TINY_INLINE static Scalar cos(const Scalar &s) {
     return TinyConstants::cos1(s);
   }
-  
+
   TINY_INLINE static Scalar acos(const Scalar &s) {
     return TinyConstants::acos(s);
   }
